<?php
use Twig\Environment;
use Twig\Loader\FilesystemLoader;

use Twig\TwigFilter;
use Twig\TwigFunction; // Updated to use TwigFunction for Twig 3.x

// Initialize the Twig loader
$loader = new FilesystemLoader([
    dirname(dirname(__FILE__)) . '/views',
    dirname(dirname(__FILE__)) . '/public',
]);

// Initialize the Twig environment
$twig = new Environment($loader, [
    'autoescape' => false,
    'debug'      => false,
]);

if ( ! is_admin() ) {

    // Add 'shortcode' filter
    $twig->addFilter(new TwigFilter('shortcode', function ( $string ) {
        return do_shortcode($string);
    }));

    // Add 'translate' filter
    $twig->addFilter(new TwigFilter('translate', function ( $string, $key ) {
        if ( ! isset($_GET['responsive-menu-pro-preview']) ) {
            $translated = apply_filters('wpml_translate_single_string', $string, 'Responsive Menu Pro', $key);
            $translated = function_exists('pll__') ? pll__($translated) : $translated;
            return $translated;
        }
        return $string;
    }));

    // Add 'build_menu' function
    $twig->addFunction(new TwigFunction('build_menu', function ( $env, $options ) {
        $translator = $env->getFilter('translate')->getCallable();
        $different_menu_for_mobile = $options['different_menu_for_mobile'];

        $menu = ($different_menu_for_mobile == 'on' && wp_is_mobile())
            ? $translator($options['menu_to_use_in_mobile'], 'menu_to_use_in_mobile')
            : $translator($options['menu_to_use'], 'menu_to_use');

        $walker = $options['custom_walker']
            ? new $options['custom_walker']($options)
            : new ResponsiveMenuPro\Walkers\Walker($options);

        // Check if menu exists
        $menu_arial_label = '';
        foreach ( get_terms('nav_menu') as $nav_menu ) {
            if ( $nav_menu->slug === $menu ) {
                $menu_arial_label = $menu;
                break;
            }
        }

        return wp_nav_menu([
            'container'      => '',
            'menu_id'        => 'responsive-menu-pro',
            'menu_class'     => null,
            'menu'           => $menu && ! $options['theme_location_menu'] ? $menu : null,
            'depth'          => $options['menu_depth'] ? $options['menu_depth'] : 0,
            'theme_location' => $options['theme_location_menu'] ? $options['theme_location_menu'] : null,
            'walker'         => $walker,
            'items_wrap'     => '<ul id="%1$s" role="menubar" aria-label="' . $menu_arial_label . '" >%3$s</ul>',
            'echo'           => false,
        ]);
    }, [ 'needs_environment' => true ]));

    // Add global variables
    $twig->addGlobal('search_url', function_exists('icl_get_home_url') ? icl_get_home_url() : get_home_url());

} else {

    // Add admin-specific functions
    $twig->addFunction(new TwigFunction('csrf', function () {
        return wp_nonce_field('update', 'responsive-menu-pro-nonce', true, false);
    }));

    $twig->addFunction(new TwigFunction('current_page', function () {
        return get_option('responsive_menu_pro_current_page', 'license');
    }));

    $twig->addFunction(new TwigFunction('license_type', function () {
        return get_option('responsive_menu_pro_license_type');
    }));

    $twig->addFunction(new TwigFunction('license_key', function () {
        return get_option('responsive_menu_pro_license_key');
    }));

    $twig->addFunction(new TwigFunction('menu_items', function ( $options ) {
        $menu = '';
        if ( $options['theme_location_menu'] ) {
            $menu = get_term(get_nav_menu_locations()[ $options['theme_location_menu'] ], 'nav_menu')->name;
        } elseif ( $options['menu_to_use'] ) {
            $menu = $options['menu_to_use'];
        } elseif ( ! empty(get_terms('nav_menu')[0]->slug) ) {
            $menu = get_terms('nav_menu')[0]->slug;
        }
        return wp_get_nav_menu_items($menu);
    }));

    $twig->addFunction(new TwigFunction('all_pages', function () {
        $posts = get_posts([
			'numberposts' => -1,
			'post_type'   => 'any',
		]);
        $all_pages = [];
        foreach ($posts as $post)
            $all_pages[ $post->ID ] = $post->post_title;
        return $all_pages;
    }));

    $twig->addFunction(new TwigFunction('font_icons', function ( $array ) {
        $new_array = [];
        for ( $i = 0; $i < count($array['id']); $i++ ) {
            $new_array[ $i ] = [
                'id'   => $array['id'][ $i ],
                'icon' => $array['icon'][ $i ],
                'type' => $array['type'][ $i ],
            ];
        }
        return $new_array;
    }));

    // Add global variables
    $twig->addGlobal('admin_url', get_admin_url());
    $twig->addGlobal('shortcode', '[responsive_menu_pro]');

    $twig->addFunction(new TwigFunction('get_available_themes', function () {
        $theme_folder_path = wp_upload_dir()['basedir'] . '/responsive-menu-themes';
        $theme_folders = glob($theme_folder_path . '/*', GLOB_ONLYDIR);

        $themes = [];
        foreach ( $theme_folders as $theme_folder ) {
            $config = json_decode(file_get_contents($theme_folder . '/config.json'), true);
            $themes[ basename($theme_folder) ]['version'] = $config['version'];
            $themes[ basename($theme_folder) ]['name'] = $config['name'];
        }

        return $themes;
    }));

    $twig->addGlobal('themes_folder_url', wp_upload_dir()['baseurl'] . '/responsive-menu-themes/');
    $twig->addGlobal('themes_folder_dir', wp_upload_dir()['basedir'] . '/responsive-menu-themes/');

    $twig->addFunction(new TwigFunction('nav_menus', function () {
        $menus_array = [];
        foreach (get_terms('nav_menu') as $menu)
            $menus_array[ $menu->slug ] = $menu->name;

        return $menus_array;
    }));

    $twig->addFunction(new TwigFunction('location_menus', function () {
        $location_menus = [ '' => 'None' ];
        foreach (get_registered_nav_menus() as $location => $menu)
            $location_menus[ $location ] = $menu;

        return $location_menus;
    }));
}

$twig->addFilter(new TwigFilter('json_decode', function ( $string ) {
    if ( is_string($string) ) {
        return json_decode($string, true);
    }
    return null;
}));

// Add 'header_bar_items' function
$twig->addFunction(new TwigFunction('header_bar_items', function ( $items ) {
    if (isset($items['button']))
        unset($items['button']);
    return $items;
}));

return $twig;